@@ -1,6 +1,5 @@
 use super::*;
 use russell_openblas::*;
-use std::convert::TryInto;
 
 /// Performs the inner (dot) product between two vectors resulting in a scalar value
 ///
@@ -10,9 +9,9 @@ use std::convert::TryInto;
 ///
 /// # Note
 ///
-/// The lengths of both vectors may be different; the smaller length will be selected.
+/// The lengths of both vectors may be different; the smallest length will be selected.
 ///
-/// # Examples
+/// # Example
 ///
 /// ```
 /// use russell_lab::*;
@@ -23,12 +22,9 @@ use std::convert::TryInto;
 /// ```
 ///
 pub fn inner(u: &Vector, v: &Vector) -> f64 {
-    let n = if u.data.len() < v.data.len() {
-        u.data.len()
-    } else {
-        v.data.len()
-    };
-    ddot(n.try_into().unwrap(), &u.data, 1, &v.data, 1)
+    let n = if u.dim() < v.dim() { u.dim() } else { v.dim() };
+    let n_i32 = to_i32(n);
+    ddot(n_i32, u.as_data(), 1, v.as_data(), 1)
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -44,4 +40,12 @@ mod tests {
         let y = Vector::from(&[-15.0, -5.0, -24.0]);
         assert_eq!(inner(&x, &y), -1070.0);
     }
+
+    #[test]
+    fn inner_alt_works() {
+        const IGNORED: f64 = 100000.0;
+        let x = Vector::from(&[-15.0, -5.0, -24.0]);
+        let y = Vector::from(&[20.0, 10.0, 30.0, IGNORED]);
+        assert_eq!(inner(&x, &y), -1070.0);
+    }
 }
